package com.nisum.mytime.controller;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.nisum.mytime.exception.handler.MyTimeException;
import com.nisum.mytime.model.Billing;
import com.nisum.mytime.model.EmployeeDashboardVO;
import com.nisum.mytime.model.Employee;
import com.nisum.mytime.model.EmployeeVisa;
import com.nisum.mytime.model.Project;
import com.nisum.mytime.model.Resource;
import com.nisum.mytime.repository.EmployeeVisaRepo;
import com.nisum.mytime.service.IProjectService;
import com.nisum.mytime.service.IEmployeeService;
import com.nisum.mytime.utils.MyTimeUtils;

@RestController
@RequestMapping("/projectTeam")
public class ProjectTeamController {

    @Autowired
    private IEmployeeService userService;

    @Autowired
    private IProjectService projectService;

    @Autowired
    private EmployeeVisaRepo employeeVisaRepo;
    
    @RequestMapping(value = "/employee", method = RequestMethod.GET,
            produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<Employee> getEmployeeRole(
            @RequestParam("emailId") String emailId) throws MyTimeException {
        Employee employeesRole = userService.getEmployeeByEmaillId(emailId);
        return new ResponseEntity<>(employeesRole, HttpStatus.OK);
    }

    @RequestMapping(value = "/addProject", method = RequestMethod.POST,
            produces = MediaType.APPLICATION_JSON_VALUE,
            consumes = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<Project> addProject(
            @RequestBody Project projectObj, @RequestParam(value="loginEmpId") String  loginEmpId) throws MyTimeException {
        Project project = projectService.addProject(projectObj, loginEmpId);
        return new ResponseEntity<>(project, HttpStatus.OK);
    }

    @RequestMapping(value = "/updateEmployeeRole", method = RequestMethod.POST,
            produces = MediaType.APPLICATION_JSON_VALUE,
            consumes = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<Employee> updateEmployeeRole(@RequestBody Employee emp,
    		@RequestParam(value="empId") String  loginEmpId) throws MyTimeException {
        Employee employeeRole = userService.updateEmployee(emp, loginEmpId);
        return new ResponseEntity<>(employeeRole, HttpStatus.OK);
    }

    @RequestMapping(value = "/deleteEmployee", method = RequestMethod.DELETE,
            produces = MediaType.TEXT_PLAIN_VALUE)
    public ResponseEntity<String> deleteEmployee(
            @RequestParam("empId") String empId) throws MyTimeException {
        userService.deleteEmployee(empId);
        return new ResponseEntity<>("Success", HttpStatus.OK);
    }

    @RequestMapping(value = "/getEmployeeRoleData", method = RequestMethod.GET,
            produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<Employee> getEmployeeRoleData(
            @RequestParam("empId") String empId) throws MyTimeException {
        Employee employeesRole = userService.getEmployeeById(empId);
        return new ResponseEntity<>(employeesRole, HttpStatus.OK);
    }
    
    //MT-72
    @RequestMapping(value = "/getEmployeeProjectInfo", method = RequestMethod.GET,
            produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<List<Resource>> getEmployeeProjectInfo(
            @RequestParam("empId") String empId) throws MyTimeException {
    	List<Resource> projectInfo = projectService.getProjectInfo(empId);
        return new ResponseEntity<>(projectInfo, HttpStatus.OK);
    }
    
    

    @RequestMapping(value = "/getEmployeesToTeam", method = RequestMethod.GET,
            produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<List<Employee>> getManagers()
            throws MyTimeException {
        List<Employee> employeesRoles = new ArrayList<>();
        if (userService.getActiveEmployees() != null) {
            employeesRoles = userService.getActiveEmployees().stream()
                    .sorted((o1, o2) -> o1.getEmployeeName()
                            .compareTo(o2.getEmployeeName()))
                    .collect(Collectors.toList());
        }
        return new ResponseEntity<>(employeesRoles, HttpStatus.OK);
    }

    @RequestMapping(value = "/getTeamDetails", method = RequestMethod.GET,
            produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<List<Resource>> getTeamDetails(
            @RequestParam("employeeId") String employeeId)
            throws MyTimeException {
        List<Resource> employeesRoles = projectService
                .getTeamDetails(employeeId);
        return new ResponseEntity<>(employeesRoles, HttpStatus.OK);
    }

    @RequestMapping(value = "/addEmployeeToTeam", method = RequestMethod.POST,
            produces = MediaType.APPLICATION_JSON_VALUE,
            consumes = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<Resource> addEmployeeToTeam(
            @RequestBody Resource teamMate, @RequestParam(value="loginEmpId", required = false) String loginEmpId) throws MyTimeException {
        teamMate.setActive(true);
        teamMate.setAuditFields(loginEmpId, MyTimeUtils.CREATE);
        Resource teamMateDB = projectService.addProjectTeamMate(teamMate, loginEmpId);
        return new ResponseEntity<>(teamMateDB, HttpStatus.OK);
    }

    @RequestMapping(value = "/updateTeammate", method = RequestMethod.POST,
    		produces = MediaType.TEXT_PLAIN_VALUE,
            consumes = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<String> updateTeammate(@RequestBody Resource projectTeamMate, @RequestParam(value = "loginEmpId") String loginEmpId)
            throws MyTimeException {
    	projectTeamMate.setAuditFields(loginEmpId, MyTimeUtils.UPDATE);
    	String response = projectService.updateTeammate(projectTeamMate, loginEmpId);
        return new ResponseEntity<>(response, HttpStatus.OK);
    }

    @RequestMapping(value = "/deleteTeammate", method = RequestMethod.POST,
            produces = MediaType.TEXT_PLAIN_VALUE,
            consumes = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<String> deleteTeammate(
            @RequestBody Resource projectTeamMate, @RequestParam(value = "loginEmpId") String loginEmpId)
            throws MyTimeException {
        projectService.deleteTeammate(projectTeamMate.getEmployeeId(),
                projectTeamMate.getProjectId(), projectTeamMate.getId(), loginEmpId);
        return new ResponseEntity<>("Success", HttpStatus.OK);
    }

    @RequestMapping(value = "/getProjects", method = RequestMethod.GET,
            produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<List<Project>> getProjects(
            @RequestParam("employeeId") String employeeId)
            throws MyTimeException {
        List<Project> projects = projectService.getProjects(employeeId);
        return new ResponseEntity<>(projects, HttpStatus.OK);
    }

    @RequestMapping(value = "/getMyTeamDetails", method = RequestMethod.GET,
            produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<List<Resource>> getMyTeamDetails(
            @RequestParam("employeeId") String employeeId)
            throws MyTimeException {
        List<Resource> employeesRoles = projectService
                .getMyTeamDetails(employeeId);
        return new ResponseEntity<>(employeesRoles, HttpStatus.OK);
    }

    @RequestMapping(value = "/getUnAssignedEmployees",
            method = RequestMethod.GET,
            produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<List<Employee>> getUnAssignedEmployees()
            throws MyTimeException {
        List<Employee> employeesRoles = projectService
                .getUnAssignedEmployees();
        return new ResponseEntity<>(employeesRoles, HttpStatus.OK);
    }

    @RequestMapping(value = "/getShiftDetails", method = RequestMethod.GET,
            produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<List<Resource>> getShiftDetails(
            @RequestParam("shift") String shift) throws MyTimeException {
        List<Resource> employeesRoles = projectService
                .getShiftDetails(shift);
        return new ResponseEntity<>(employeesRoles, HttpStatus.OK);
    }

    @RequestMapping(value = "/getProjectAllocations",
            method = RequestMethod.GET,
            produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<List<Resource>> getProjectAllocations()
            throws MyTimeException {
        List<Resource> employeesRoles = projectService
                .getAllProjectDetails();
        return new ResponseEntity<>(employeesRoles, HttpStatus.OK);
    }

    @RequestMapping(value = "/getProjectDetails", method = RequestMethod.GET,
            produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<List<Resource>> getProjectDetails(
            @RequestParam("projectId") String projectId,
            @RequestParam(value = "status", required = false, defaultValue  = MyTimeUtils.ACTIVE) String status)
            throws MyTimeException {
        List<Resource> employeesRoles = projectService
                .getProjectDetails(projectId,status);
        return new ResponseEntity<>(employeesRoles, HttpStatus.OK);
    }

    @RequestMapping(value = "/getMyProjectAllocations",
            method = RequestMethod.GET,
            produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<List<HashMap<Object, Object>>> getMyProjectAllocations(
            @RequestParam("employeeId") String employeeId)
            throws MyTimeException {
       // List<ProjectTeamMate> projectAllocations = projectService
       //         .getMyProjectAllocations(employeeId);
        List<HashMap<Object, Object>> empPrjtsInfo = projectService.projectsInfoByEmpId(employeeId);
        return new ResponseEntity<>(empPrjtsInfo, HttpStatus.OK);
    }

    @RequestMapping(value = "/getEmployeeBillingDetails",
            method = RequestMethod.GET,
            produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<List<Billing>> getEmployeeBillingDetails(
            @RequestParam("employeeId") String employeeId,
            @RequestParam("projectId") String projectId)
            throws MyTimeException {
        List<Billing> billings = projectService
                .getEmployeeBillingDetails(employeeId, projectId);
        return new ResponseEntity<>(billings, HttpStatus.OK);
    }

    @RequestMapping(value = "/getEmployeeBillingDetailsAll",
            method = RequestMethod.GET,
            produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<List<Billing>> getEmployeeBillingDetailsAll(
            @RequestParam("employeeId") String employeeId)
            throws MyTimeException {
        List<Billing> billings = projectService
                .getEmployeeBillingDetailsAll(employeeId);
        return new ResponseEntity<>(billings, HttpStatus.OK);
    }

    @RequestMapping(value = "/addEmployeeBilling", method = RequestMethod.POST,
            produces = MediaType.APPLICATION_JSON_VALUE,
            consumes = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<Billing> addEmployeeBilling(
            @RequestBody Billing billingDetails, @RequestParam(value = "loginEmpId") String loginEmpId) throws MyTimeException {
        Billing billings = projectService
                .addEmployeeBillingDetails(billingDetails, loginEmpId);

        return new ResponseEntity<>(billings, HttpStatus.OK);
    }

    @RequestMapping(value = "/updateEmployeeBilling",
            method = RequestMethod.POST,
            produces = MediaType.APPLICATION_JSON_VALUE,
            consumes = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<Billing> updateEmployeeBilling(
            @RequestBody Billing billingDetails, @RequestParam(value = "loginEmpId") String loginEmpId) throws MyTimeException {
        Billing billings = projectService
                .updateEmployeeBilling(billingDetails, loginEmpId);
        return new ResponseEntity<>(billings, HttpStatus.OK);
    }

    @RequestMapping(value = "/deleteEmployeeBilling",
            method = RequestMethod.POST,
            produces = MediaType.APPLICATION_JSON_VALUE,
            consumes = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<Billing> deleteEmployeeBilling(
            @RequestBody Billing teamMate) throws MyTimeException {
        projectService.deleteEmployeeBilling(teamMate);
        return new ResponseEntity<>(null, HttpStatus.OK);
    }

    @RequestMapping(value = "/getEmployeesDashBoard",
            method = RequestMethod.GET,
            produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<List<EmployeeDashboardVO>> getEmployeesDashBoard()
            throws MyTimeException {
        List<EmployeeDashboardVO> employeesRoles = projectService
                .getEmployeesDashBoard();
        return new ResponseEntity<>(employeesRoles, HttpStatus.OK);
    }

    @RequestMapping(value = "/getEmployeesHavingVisa",
            method = RequestMethod.GET,
            produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<List<Employee>> getEmployeesHavingVisa(
            @RequestParam("visa") String passport) throws MyTimeException {
    	 List<Employee> employees = new ArrayList<>();
        if (passport != null && !"passport".equalsIgnoreCase(passport)) {
            List<EmployeeVisa> employeeVisas = employeeVisaRepo
                    .findByVisaName(passport);
            List<String> employeeIds = null;
            if (employeeVisas != null) {
                employeeIds = employeeVisas.stream()
                        .map(EmployeeVisa::getEmployeeId)
                        .collect(Collectors.toList());
            }
            if (employeeIds != null && !employeeIds.isEmpty()) {
                List<Employee> emps = userService.getActiveEmployees();
                for (Employee emp : emps) {
                    if (employeeIds.contains(emp.getEmployeeId())) {
                        employees.add(emp);
                    }
                }
            }
            return new ResponseEntity<>(employees, HttpStatus.OK);
        } else {
            //List<EmployeeRoles> employees = new ArrayList<>();
            if (userService.getActiveEmployees() != null) {
                employees = userService.getActiveEmployees().stream()
                        .sorted((o1, o2) -> o1.getEmployeeName()
                                .compareTo(o2.getEmployeeName()))
                        .collect(Collectors.toList());
            }
            return new ResponseEntity<>(employees, HttpStatus.OK);
        }

    }
    
    @RequestMapping(value = "/addEmployeeToTeamWithCheck", method = RequestMethod.POST,
            produces = MediaType.TEXT_PLAIN_VALUE,
            consumes = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<String> addEmployeeToTeamWithCheck(
            @RequestBody Resource teamMate, @RequestParam(value = "loginEmpId") String loginEmpId) throws MyTimeException {
    	String response = projectService.addProjectTeamMateWithCheck(teamMate, loginEmpId);
    	return new ResponseEntity<>(response, HttpStatus.OK); 
    }
}