package com.nisum.mytime.service.impl;

import java.util.Date;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.nisum.mytime.model.Billing;
import com.nisum.mytime.model.Employee;
import com.nisum.mytime.model.Resource;
import com.nisum.mytime.repository.ResourceRepo;
import com.nisum.mytime.service.IResourceService;
import com.nisum.mytime.utils.MyTimeUtils;

import lombok.extern.slf4j.Slf4j;

@Service
@Slf4j
public class ResourceService implements IResourceService {

	@Autowired
	private BillingService billingService;

	@Autowired
	private EmployeeShiftService empShiftService;

	@Autowired
	private ResourceRepo resourceRepo;

	public Resource save(Resource resource) {
		return resourceRepo.save(resource);
	}

	public void addResources(Employee employee, String loginEmpId) {
		List<Resource> resourceList = resourceRepo.findByEmployeeId(employee.getEmployeeId());
		if (resourceList != null && !resourceList.isEmpty()) {
			for (Resource resource : resourceList) {
				resource.setRole(employee.getRole());
				resource.setAuditFields(loginEmpId, MyTimeUtils.UPDATE);
				resourceRepo.save(resource);
			}
		}

		// return null;
	}

	public void inactivateResource(Employee employeeReq, Employee employeeUpdated, String loginEmpId) {
		// inactive the employee from the assigned project
		if (employeeReq.getEmpStatus().equals(MyTimeUtils.IN_ACTIVE_SPACE) && employeeReq.getEndDate() != null
				&& employeeReq.getEndDate().compareTo(new Date()) <= 0) {

			List<Resource> resourcesList = resourceRepo.findByEmployeeIdAndActive(employeeReq.getEmployeeId(), true);
			if (resourcesList.size() == 1) {
				Resource resource = resourcesList.get(0);
				resource.setActive(false);
				resource.setEndDate(employeeReq.getEndDate());

				List<Billing> billingList = billingService.getEmployeeActiveBillingDetails(resource.getEmployeeId(),
						resource.getProjectId());
				if (billingList != null && !billingList.isEmpty()) {
					billingService.addBillingtoResource(billingList.get(0), employeeUpdated, loginEmpId);
				}
				resource.setAuditFields(loginEmpId, MyTimeUtils.UPDATE);
				resourceRepo.save(resource);
				empShiftService.updateShiftDetails(resource, loginEmpId);
			}
		}
	}

	public List<Resource> getResources(String empId) {
		return resourceRepo.findByEmployeeId(empId);
	}

}
